<?php
/**
 * Blocks Initializer
 *
 * Enqueue CSS/JS of all the blocks.
 *
 * @since   1.0.0
 *
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class plusbf
{
	function __construct()
	{
		add_action( 'wp_head', array( $this, 'render_server_side_css') );
		add_action( 'init', array( $this, 'block_assets' ) );
		add_action( 'wp_loaded', array( $this, 'add_attributes_to_blocks' ) );
	}

	public function block_assets() {

		wp_register_script(
			'plusbf-block-js', // Handle.
			plugins_url( '/dist/blocks.build.js', dirname( __FILE__ ) ),
			array( 'wp-blocks', 'wp-i18n', 'wp-element', 'wp-editor' ),
			null,
			true
		);

		wp_register_style(
			'plusbf-block-editor-css', // Handle.
			plugins_url( 'dist/blocks.editor.build.css', dirname( __FILE__ ) ),
			array( 'wp-edit-blocks' ),
			null
		);

		wp_localize_script(
			'plusbf-block-js',
			'plusbfGlobal',
			[
				'pluginDirPath'	=> plugin_dir_path( __DIR__ ),
				'pluginDirUrl'	=> plugin_dir_url( __DIR__ ),
			]
		);

		register_block_type(
			'plusbf/block-style', array(
				'editor_script' => 'plusbf-block-js',
				'editor_style'  => 'plusbf-block-editor-css',
			)
		);
	}

	public function add_attributes_to_blocks() {
		$regist_blocks = \WP_Block_Type_Registry::get_instance()
			->get_all_registered();

		foreach ( $regist_blocks as $name => $block ) {
			$block->attributes['plusbfClassName'] = [
				'type'    => 'string',
				'default' => '',
			];
			$block->attributes['plusbfStyle'] = [
				'type'    => 'string',
				'default' => '',
			];
		}
	}

	public function build_style( $blocks )
	{
		$styles = [];
		foreach ($blocks as $block) {
			if ( isset( $block['attrs'] ) ) {
				if ( isset( $block['attrs']['plusbfStyle'] ) ) {
					$styles[] = $block['attrs']['plusbfStyle'];
				}
			}

			//再利用ブロックのとき
			if ('core/block' === $block['blockName'] && isset( $block['attrs']['ref'] ) && intval( $block['attrs']['ref'] ) > 0) {
				$reusable_block = get_post( $block['attrs']['ref'] );
				//idに該当するpostがない
				//postはあってもwp_blockではない
				//postが公開ではない
				//ときは非表示
				if (!$reusable_block ||
					'wp_block' !== $reusable_block->post_type ||
					'publish' !== $reusable_block->post_status
				) {
					continue;
				}
				$reusable_inner_blocks = $this->parse_blocks( $reusable_block->post_content );
				$styles[] = $this->build_style( $reusable_inner_blocks );
			}

			if ( isset( $block['innerBlocks'] ) && !empty( $block['innerBlocks'] ) && is_array( $block['innerBlocks'] )) {
				$styles[] = $this->build_style( $block['innerBlocks'] );
			}
		}
		return implode("\n", $styles);
	}

	public function parse_blocks($content)
	{
		if (function_exists('parse_blocks')) {
			return parse_blocks($content);
		} else {
			return gutenberg_parse_blocks($content);
		}
	}

	public function render_server_side_css()
	{
		if ( function_exists('has_blocks') && has_blocks( get_the_ID() ) ) {
			global $post;

			if (!is_object($post)) {
				return;
			}

			$blocks = $this->parse_blocks( $post->post_content );

			if (!is_array( $blocks ) || empty( $blocks )) {
				return;
			}
			echo '<style type="text/css">';
			echo htmlspecialchars($this->build_style($blocks), ENT_NOQUOTES);
			echo '</style>';
		}
	}
}

$plusbf_ins = new plusbf;
